@@ -6,15 +6,53 @@ use crate::{
         AddAttributes, AppendChildren, ChangeText, RemoveAttributes,
         RemoveNode, ReplaceNode,
     },
-    Attribute, Element, Node, Patch,
+    Attribute, Element, Node, Patch, PatchPath, TreePath,
 };
 use keyed_elements::diff_keyed_elements;
 use std::fmt::Debug;
-use std::{cmp, fmt, mem};
+use std::{cmp, mem};
 
 mod keyed_elements;
 
-/// calculate the difference of 2 nodes
+/// Return the patches needed for `old_node` to have the same DOM as `new_node`
+///
+/// # Agruments
+/// * old_node - the old virtual dom node
+/// * new_node - the new virtual dom node
+/// * key - the literal name of key attribute, ie: "key"
+///
+/// # Example
+/// ```rust
+/// use mt_dom::{diff::*, patch::*, *};
+///
+/// pub type MyNode =
+///    Node<&'static str, &'static str, &'static str, &'static str, ()>;
+///
+/// let old: MyNode = element(
+///     "main",
+///     vec![attr("class", "container")],
+///     vec![
+///         element("div", vec![attr("key", "1")], vec![]),
+///         element("div", vec![attr("key", "2")], vec![]),
+///     ],
+/// );
+///
+/// let new: MyNode = element(
+///     "main",
+///     vec![attr("class", "container")],
+///     vec![element("div", vec![attr("key", "2")], vec![])],
+/// );
+///
+/// let diff = diff_with_key(&old, &new, &"key");
+/// assert_eq!(
+///     diff,
+///     vec![RemoveNode::new(
+///         Some(&"div"),
+///         PatchPath::old(TreePath::start_at(1, vec![0, 0]),),
+///     )
+///     .into()]
+/// );
+/// ```
 pub fn diff_with_key<'a, NS, TAG, ATT, VAL, EVENT>(
     old_node: &'a Node<NS, TAG, ATT, VAL, EVENT>,
     new_node: &'a Node<NS, TAG, ATT, VAL, EVENT>,
@@ -32,6 +70,8 @@ where
         new_node,
         &mut 0,
         &mut 0,
+        &vec![0],
+        &vec![0],
         key,
         &|_old, _new| false,
         &|_old, _new| false,
@@ -72,16 +112,24 @@ where
         &'a Node<NS, TAG, ATT, VAL, EVENT>,
     ) -> bool,
 {
-    diff_recursive(old_node, new_node, &mut 0, &mut 0, key, skip, rep)
+    diff_recursive(
+        old_node,
+        new_node,
+        &mut 0,
+        &mut 0,
+        &vec![0],
+        &vec![0],
+        key,
+        skip,
+        rep,
+    )
 }
 
 /// increment the cur_node_idx based on how many descendant it contains.
 ///
 /// Note: This is not including the count of itself, since the node is being processed and the cur_node_idx is
 /// incremented in the loop together with its siblings
-/// TODO: this can be optimize by adding the children count
-/// and then descending into element node that has children only
-pub fn increment_node_idx_to_descendant_count<NS, TAG, ATT, VAL, EVENT>(
+fn increment_node_idx_to_descendant_count<NS, TAG, ATT, VAL, EVENT>(
     node: &Node<NS, TAG, ATT, VAL, EVENT>,
     cur_node_idx: &mut usize,
 ) where
@@ -141,11 +189,63 @@ where
     }
 }
 
+fn should_replace<'a, 'b, NS, TAG, ATT, VAL, EVENT, REP>(
+    old_node: &'a Node<NS, TAG, ATT, VAL, EVENT>,
+    new_node: &'a Node<NS, TAG, ATT, VAL, EVENT>,
+    key: &ATT,
+    rep: &REP,
+) -> bool
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+    EVENT: PartialEq + Clone + Debug,
+    REP: Fn(
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+    ) -> bool,
+{
+    if mem::discriminant(old_node) != mem::discriminant(new_node) {
+        return true;
+    }
+
+    // handle explicit replace if the REP fn evaluates to true
+    if rep(old_node, new_node) {
+        return true;
+    }
+
+    // replace if the old key does not match the new key
+    match (
+        old_node.get_attribute_value(&key),
+        new_node.get_attribute_value(&key),
+    ) {
+        (Some(old_key), Some(new_key)) => {
+            if old_key != new_key {
+                return true;
+            }
+        }
+        _ => (),
+    }
+    // Different enum variants, replace!
+    if let (Node::Element(old_element), Node::Element(new_element)) =
+        (old_node, new_node)
+    {
+        // Replace if there are different element tags
+        if old_element.tag != new_element.tag {
+            return true;
+        }
+    }
+    false
+}
+
 fn diff_recursive<'a, 'b, NS, TAG, ATT, VAL, EVENT, SKIP, REP>(
     old_node: &'a Node<NS, TAG, ATT, VAL, EVENT>,
     new_node: &'a Node<NS, TAG, ATT, VAL, EVENT>,
     cur_node_idx: &'b mut usize,
     new_node_idx: &'b mut usize,
+    cur_path: &Vec<usize>,
+    new_path: &Vec<usize>,
     key: &ATT,
     skip: &SKIP,
     rep: &REP,
@@ -172,46 +272,19 @@ where
         return vec![];
     }
 
-    let mut replace =
-        mem::discriminant(old_node) != mem::discriminant(new_node);
+    let replace = should_replace(old_node, new_node, key, rep);
 
     let mut patches = vec![];
 
-    // handle explicit replace if the REP fn evaluates to true
-    if rep(old_node, new_node) {
-        replace = true;
-    }
-
-    // replace if the old key does not match the new key
-    match (
-        old_node.get_attribute_value(&key),
-        new_node.get_attribute_value(&key),
-    ) {
-        (Some(old_key), Some(new_key)) => {
-            if old_key != new_key {
-                replace = true;
-            }
-        }
-        _ => (),
-    }
-    // Different enum variants, replace!
-    if let (Node::Element(old_element), Node::Element(new_element)) =
-        (old_node, new_node)
-    {
-        // Replace if there are different element tags
-        if old_element.tag != new_element.tag {
-            replace = true;
-        }
-    }
-
-    // Handle implicit replace
     if replace {
         patches.push(
             ReplaceNode::new(
                 old_node.tag(),
-                *cur_node_idx,
-                *new_node_idx,
-                &new_node,
+                PatchPath::new(
+                    TreePath::start_at(*cur_node_idx, cur_path.clone()),
+                    TreePath::start_at(*new_node_idx, new_path.clone()),
+                ),
+                new_node,
             )
             .into(),
         );
@@ -228,9 +301,11 @@ where
         (Node::Text(old_text), Node::Text(new_text)) => {
             if old_text != new_text {
                 let ct = ChangeText::new(
-                    *cur_node_idx,
                     old_text,
-                    *new_node_idx,
+                    PatchPath::new(
+                        TreePath::start_at(*cur_node_idx, cur_path.clone()),
+                        TreePath::start_at(*new_node_idx, new_path.clone()),
+                    ),
                     new_text,
                 );
                 patches.push(Patch::ChangeText(ct));
@@ -250,6 +325,8 @@ where
                     key,
                     cur_node_idx,
                     new_node_idx,
+                    cur_path,
+                    new_path,
                     skip,
                     rep,
                 );
@@ -261,6 +338,8 @@ where
                     key,
                     cur_node_idx,
                     new_node_idx,
+                    cur_path,
+                    new_path,
                     skip,
                     rep,
                 );
@@ -294,6 +373,8 @@ fn diff_non_keyed_elements<'a, 'b, NS, TAG, ATT, VAL, EVENT, SKIP, REP>(
     key: &ATT,
     cur_node_idx: &'b mut usize,
     new_node_idx: &'b mut usize,
+    cur_path: &Vec<usize>,
+    new_path: &Vec<usize>,
     skip: &SKIP,
     rep: &REP,
 ) -> Vec<Patch<'a, NS, TAG, ATT, VAL, EVENT>>
@@ -313,9 +394,16 @@ where
     ) -> bool,
 {
     let this_cur_node_idx = *cur_node_idx;
+    let this_cur_path = cur_path.clone();
     let mut patches = vec![];
-    let attributes_patches =
-        diff_attributes(old_element, new_element, cur_node_idx, new_node_idx);
+    let attributes_patches = create_attribute_patches(
+        old_element,
+        new_element,
+        *cur_node_idx,
+        *new_node_idx,
+        cur_path,
+        new_path,
+    );
     patches.extend(attributes_patches);
 
     let old_child_count = old_element.children.len();
@@ -326,6 +414,15 @@ where
         *cur_node_idx += 1;
         *new_node_idx += 1;
 
+        let mut cur_child_path = cur_path.clone();
+        let mut new_child_path = new_path.clone();
+        cur_child_path.push(index);
+        println!(
+            "\t we just added index: {} ... cur_child_path is now: {:?}",
+            index, cur_child_path
+        );
+        new_child_path.push(index);
+
         let old_child = &old_element
             .children
             .get(index)
@@ -338,59 +435,122 @@ where
             new_child,
             cur_node_idx,
             new_node_idx,
+            &mut cur_child_path,
+            &mut new_child_path,
             key,
             skip,
             rep,
         );
         patches.extend(more_patches);
-        //increment_node_idx_to_descendant_count(new_child, new_node_idx);
     }
 
     // If there are more new child than old_node child, we make a patch to append the excess element
     // starting from old_child_count to the last item of the new_elements
     if new_child_count > old_child_count {
-        let mut append_patch: Vec<(usize, &'a Node<NS, TAG, ATT, VAL, EVENT>)> =
-            vec![];
-
-        for append_child in new_element.children.iter().skip(old_child_count) {
-            *new_node_idx += 1;
-            append_patch.push((*new_node_idx, append_child));
-            increment_node_idx_to_descendant_count(append_child, new_node_idx);
-        }
-
-        patches.push(
-            AppendChildren::new(
-                &old_element.tag,
-                this_cur_node_idx,
-                append_patch,
-            )
-            .into(),
-        )
+        let append_children_patch = create_append_children_patch(
+            old_element,
+            new_element,
+            this_cur_node_idx,
+            new_node_idx,
+            this_cur_path,
+        );
+        patches.push(append_children_patch);
     }
 
     if new_child_count < old_child_count {
-        for old_child in old_element.get_children().iter().skip(new_child_count)
-        {
-            *cur_node_idx += 1;
-            patches
-                .push(RemoveNode::new(old_child.tag(), *cur_node_idx).into());
-            increment_node_idx_to_descendant_count(old_child, cur_node_idx);
-        }
+        let remove_node_patches = create_remove_node_patch(
+            old_element,
+            new_element,
+            cur_node_idx,
+            cur_path,
+        );
+        patches.extend(remove_node_patches);
     }
 
     patches
 }
 
+fn create_append_children_patch<'a, 'b, NS, TAG, ATT, VAL, EVENT>(
+    old_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    new_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    this_cur_node_idx: usize,
+    new_node_idx: &'b mut usize,
+    this_cur_path: Vec<usize>,
+) -> Patch<'a, NS, TAG, ATT, VAL, EVENT>
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+    EVENT: PartialEq + Clone + Debug,
+{
+    let old_child_count = old_element.children.len();
+    let mut append_patch: Vec<(usize, &'a Node<NS, TAG, ATT, VAL, EVENT>)> =
+        vec![];
+
+    for append_child in new_element.children.iter().skip(old_child_count) {
+        *new_node_idx += 1;
+        append_patch.push((*new_node_idx, append_child));
+        increment_node_idx_to_descendant_count(append_child, new_node_idx);
+    }
+
+    AppendChildren::new(
+        &old_element.tag,
+        PatchPath::old(TreePath::start_at(
+            this_cur_node_idx,
+            this_cur_path.clone(),
+        )),
+        append_patch,
+    )
+    .into()
+}
+
+fn create_remove_node_patch<'a, 'b, NS, TAG, ATT, VAL, EVENT>(
+    old_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    new_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    cur_node_idx: &'b mut usize,
+    cur_path: &Vec<usize>,
+) -> Vec<Patch<'a, NS, TAG, ATT, VAL, EVENT>>
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+    EVENT: PartialEq + Clone + Debug,
+{
+    let new_child_count = new_element.children.len();
+    let mut patches = vec![];
+    for (i, old_child) in old_element
+        .get_children()
+        .iter()
+        .skip(new_child_count)
+        .enumerate()
+    {
+        *cur_node_idx += 1;
+        let mut child_cur_path = cur_path.clone();
+        child_cur_path.push(new_child_count + i);
+        let remove_node_patch = RemoveNode::new(
+            old_child.tag(),
+            PatchPath::old(TreePath::start_at(*cur_node_idx, child_cur_path)),
+        );
+        patches.push(remove_node_patch.into());
+        increment_node_idx_to_descendant_count(old_child, cur_node_idx);
+    }
+    patches
+}
+
 /// diff the attributes of old_node element to the new element at this cur_node_idx
 ///
 /// Note: The performance bottlenecks
 ///     - allocating new vec
 ///     - merging attributes of the same name
-fn diff_attributes<'a, 'b, NS, TAG, ATT, VAL, EVENT>(
+fn create_attribute_patches<'a, 'b, NS, TAG, ATT, VAL, EVENT>(
     old_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
     new_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
-    cur_node_idx: &'b mut usize,
-    new_node_idx: &'b mut usize,
+    cur_node_idx: usize,
+    new_node_idx: usize,
+    cur_path: &Vec<usize>,
+    new_path: &Vec<usize>,
 ) -> Vec<Patch<'a, NS, TAG, ATT, VAL, EVENT>>
 where
     NS: PartialEq + Clone + Debug,
@@ -456,8 +616,10 @@ where
         patches.push(
             AddAttributes::new(
                 &old_element.tag,
-                *cur_node_idx,
-                *new_node_idx,
+                PatchPath::new(
+                    TreePath::start_at(cur_node_idx, cur_path.clone()),
+                    TreePath::start_at(new_node_idx, new_path.clone()),
+                ),
                 add_attributes,
             )
             .into(),
@@ -467,8 +629,10 @@ where
         patches.push(
             RemoveAttributes::new(
                 &old_element.tag,
-                *cur_node_idx,
-                *new_node_idx,
+                PatchPath::new(
+                    TreePath::start_at(cur_node_idx, cur_path.clone()),
+                    TreePath::start_at(new_node_idx, new_path.clone()),
+                ),
                 remove_attributes,
             )
             .into(),
