@@ -1,12 +1,13 @@
 use super::{
-    diff_attributes, diff_recursive, increment_node_idx_to_descendant_count,
+    create_attribute_patches, diff_recursive,
+    increment_node_idx_to_descendant_count,
 };
 use crate::{
     patch::{AppendChildren, InsertNode, RemoveNode},
-    Element, Node, NodeIdx, Patch,
+    Element, Node, NodeIdx, Patch, PatchPath, TreePath,
 };
 use std::fmt::Debug;
-use std::{collections::BTreeMap, fmt, iter::FromIterator};
+use std::{collections::BTreeMap, iter::FromIterator};
 
 /// find the element and its node_idx which has this key
 /// and its node_idx not in `not_in`
@@ -135,104 +136,52 @@ where
         .collect()
 }
 
-/// Reconciliation of keyed elements
-///
-/// algorithm:
-///
-/// Phase1
-///   - prioritize matching elements that has the same key.
-///   - match non-keyed elements according to their node_idx alignment
-///
-/// Phase2
-///   - old child elements that are not matched will be removed
-///   - new child elements that are not matched will be inserted or appended
-///     - inserted if the child node_idx <= old elements children
-///     - appended if the child node_idx is > old elements children
-///
-pub fn diff_keyed_elements<'a, 'b, NS, TAG, ATT, VAL, EVENT, SKIP, REP>(
-    old_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
-    new_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+fn build_keyed_elements<'a, NS, TAG, ATT, VAL, EVENT>(
+    element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
     key: &ATT,
-    cur_node_idx: &'b mut usize,
-    new_element_node_idx: &'b mut usize,
-    skip: &SKIP,
-    rep: &REP,
-) -> Vec<Patch<'a, NS, TAG, ATT, VAL, EVENT>>
+) -> BTreeMap<usize, (Vec<&'a VAL>, &'a Node<NS, TAG, ATT, VAL, EVENT>)>
 where
     NS: PartialEq + Clone + Debug,
     TAG: PartialEq + Clone + Debug,
     ATT: PartialEq + Clone + Debug,
     VAL: PartialEq + Clone + Debug,
     EVENT: PartialEq + Clone + Debug,
-    SKIP: Fn(
-        &'a Node<NS, TAG, ATT, VAL, EVENT>,
-        &'a Node<NS, TAG, ATT, VAL, EVENT>,
-    ) -> bool,
-    REP: Fn(
-        &'a Node<NS, TAG, ATT, VAL, EVENT>,
-        &'a Node<NS, TAG, ATT, VAL, EVENT>,
-    ) -> bool,
 {
-    let mut patches = vec![];
+    BTreeMap::from_iter(element.get_children().iter().enumerate().filter_map(
+        |(idx, child)| {
+            if let Some(child_key) = child.get_attribute_value(key) {
+                Some((idx, (child_key, child)))
+            } else {
+                None
+            }
+        },
+    ))
+}
 
-    let attributes_patches = diff_attributes(
-        old_element,
-        new_element,
-        cur_node_idx,
-        new_element_node_idx,
-    );
-    // create a map for both the old and new element
-    // we can not use VAL as the key, since it is not Hash
-    let old_keyed_elements: BTreeMap<
+fn build_matched_old_new_keyed<'a, NS, TAG, ATT, VAL, EVENT>(
+    old_keyed_elements: &BTreeMap<
         usize,
-        (Vec<&VAL>, &Node<NS, TAG, ATT, VAL, EVENT>),
-    > = BTreeMap::from_iter(
-        old_element.get_children().iter().enumerate().filter_map(
-            |(old_idx, old_child)| {
-                if let Some(old_key) = old_child.get_attribute_value(key) {
-                    Some((old_idx, (old_key, old_child)))
-                } else {
-                    None
-                }
-            },
-        ),
-    );
-
-    let mut node_idx_new_elements: BTreeMap<
-        NodeIdx,
-        &Node<NS, TAG, ATT, VAL, EVENT>,
-    > = BTreeMap::new();
-
-    for new_child in new_element.get_children().iter() {
-        *new_element_node_idx += 1;
-        node_idx_new_elements.insert(*new_element_node_idx, new_child);
-        increment_node_idx_to_descendant_count(new_child, new_element_node_idx);
-    }
-
-    let mut new_keyed_elements: BTreeMap<
+        (Vec<&'a VAL>, &'a Node<NS, TAG, ATT, VAL, EVENT>),
+    >,
+    new_keyed_elements: &BTreeMap<
         usize,
-        (Vec<&VAL>, (NodeIdx, &Node<NS, TAG, ATT, VAL, EVENT>)),
-    > = BTreeMap::new();
-
-    for (new_idx, (new_node_idx, new_child)) in
-        node_idx_new_elements.iter().enumerate()
-    {
-        if let Some(new_key) = new_child.get_attribute_value(key) {
-            new_keyed_elements
-                .insert(new_idx, (new_key, (*new_node_idx, new_child)));
-        }
-    }
-
-    // compiles that matched old and new with
-    // with their (old_idx, new_idx) as key and the value is (old_element, new_element)
-    let mut matched_old_new_keyed: BTreeMap<
-        (usize, usize),
-        (
-            &Node<NS, TAG, ATT, VAL, EVENT>,
-            (NodeIdx, &Node<NS, TAG, ATT, VAL, EVENT>),
-        ),
-    > = BTreeMap::new();
-
+        (Vec<&'a VAL>, (NodeIdx, &'a Node<NS, TAG, ATT, VAL, EVENT>)),
+    >,
+) -> BTreeMap<
+    (usize, usize),
+    (
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+        (NodeIdx, &'a Node<NS, TAG, ATT, VAL, EVENT>),
+    ),
+>
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+    EVENT: PartialEq + Clone + Debug,
+{
+    let mut matched_old_new_keyed = BTreeMap::new();
     let mut last_matched_old_idx = None;
     let mut last_matched_new_idx = None;
     // here, we need to processed both keyed element and non-keyed elements
@@ -258,6 +207,96 @@ where
             }
         }
     }
+    matched_old_new_keyed
+}
+
+fn build_node_idx_new_elements<'a, NS, TAG, ATT, VAL, EVENT>(
+    new_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    new_node_idx: &mut usize,
+) -> BTreeMap<NodeIdx, &'a Node<NS, TAG, ATT, VAL, EVENT>>
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+    EVENT: PartialEq + Clone + Debug,
+{
+    let mut node_idx_new_elements = BTreeMap::new();
+
+    for new_child in new_element.get_children().iter() {
+        *new_node_idx += 1;
+        node_idx_new_elements.insert(*new_node_idx, new_child);
+        increment_node_idx_to_descendant_count(new_child, new_node_idx);
+    }
+    node_idx_new_elements
+}
+
+/// Reconciliation of keyed elements
+///
+/// algorithm:
+///
+/// Phase1
+///   - prioritize matching elements that has the same key.
+///   - match non-keyed elements according to their node_idx alignment
+///
+/// Phase2
+///   - old child elements that are not matched will be removed
+///   - new child elements that are not matched will be inserted or appended
+///     - inserted if the child node_idx <= old elements children
+///     - appended if the child node_idx is > old elements children
+///
+pub fn diff_keyed_elements<'a, 'b, NS, TAG, ATT, VAL, EVENT, SKIP, REP>(
+    old_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    new_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    key: &ATT,
+    cur_node_idx: &'b mut usize,
+    new_node_idx: &'b mut usize,
+    cur_path: &Vec<usize>,
+    new_path: &Vec<usize>,
+    skip: &SKIP,
+    rep: &REP,
+) -> Vec<Patch<'a, NS, TAG, ATT, VAL, EVENT>>
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+    EVENT: PartialEq + Clone + Debug,
+    SKIP: Fn(
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+    ) -> bool,
+    REP: Fn(
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+    ) -> bool,
+{
+    let mut patches = vec![];
+
+    // create a map for both the old and new element
+    // we can not use VAL as the key, since it is not Hash
+    let old_keyed_elements = build_keyed_elements(old_element, key);
+
+    let node_idx_new_elements =
+        build_node_idx_new_elements(new_element, new_node_idx);
+
+    let new_keyed_elements: BTreeMap<
+        usize,
+        (Vec<&VAL>, (NodeIdx, &Node<NS, TAG, ATT, VAL, EVENT>)),
+    > = BTreeMap::from_iter(
+        node_idx_new_elements.iter().enumerate().filter_map(
+            |(new_idx, (new_element_node_idx, new_child))| {
+                new_child.get_attribute_value(key).map(|new_key| {
+                    (new_idx, (new_key, (*new_element_node_idx, *new_child)))
+                })
+            },
+        ),
+    );
+
+    // compiles that matched old and new with
+    // with their (old_idx, new_idx) as key and the value is (old_element, new_element)
+    let matched_old_new_keyed =
+        build_matched_old_new_keyed(&old_keyed_elements, &new_keyed_elements);
 
     // unmatched old and idx in pass 1
     let (matched_old_idx, matched_new_idx) =
@@ -306,17 +345,20 @@ where
             }),
     );
 
-    // merge both
-    // TODO: rename `matched_old_new_keyed` to `matched_old_new_all`
+    // This is named all_matched_elements
     // since it both contains keyed and the not-necessarily keyed
     // Note: not-necessarily keyed means an element could be keyed
     // but not matched using the key value, therefore will be tried
     // to be match to pass2 which only checks for aligned idx to match each other.
-    matched_old_new_keyed.extend(matched_old_new);
+    let all_matched_elements = {
+        let mut tmp = matched_old_new_keyed.clone();
+        tmp.extend(matched_old_new);
+        tmp
+    };
 
     // unmatched old and idx in pass 2
     let (_matched_old_idx_pass2, matched_new_idx_pass2) =
-        get_matched_old_new_idx(&matched_old_new_keyed);
+        get_matched_old_new_idx(&all_matched_elements);
 
     // this elements are for inserting, appending
     let mut unmatched_new_child_pass2: Vec<(
@@ -324,11 +366,15 @@ where
         NodeIdx,
         &Node<NS, TAG, ATT, VAL, EVENT>,
     )> = vec![];
-    for (idx, (new_node_idx, new_child)) in
+    for (idx, (new_element_node_idx, new_child)) in
         node_idx_new_elements.iter().enumerate()
     {
         if !matched_new_idx_pass2.contains(&idx) {
-            unmatched_new_child_pass2.push((idx, *new_node_idx, new_child));
+            unmatched_new_child_pass2.push((
+                idx,
+                *new_element_node_idx,
+                new_child,
+            ));
         }
     }
 
@@ -341,66 +387,76 @@ where
     // keeps track of new_idx that is part of the InsertNode
     let mut already_inserted = vec![];
 
-    let new_child_excess_cur_node_idx = *cur_node_idx;
+    // cur_node_idx before incrementing inside the old element iteration and recursive increment
+    // that may be called for each of this old element children nodes
+    let snapshot_cur_node_idx = *cur_node_idx;
 
     for (old_idx, old_child) in old_element.children.iter().enumerate() {
         *cur_node_idx += 1;
-        if let Some((new_idx, (new_child_node_idx, new_child))) =
-            find_matched_new_child(&matched_old_new_keyed, old_idx)
+        let mut child_cur_path = cur_path.clone();
+        child_cur_path.push(old_idx);
+
+        let mut child_new_path = new_path.clone();
+        child_new_path.push(old_idx);
+
+        if let Some((new_idx, (mut new_child_node_idx, new_child))) =
+            find_matched_new_child(&all_matched_elements, old_idx)
         {
             // insert unmatched new_child that is less than the matched new_idx
-            for (idx, new_node_idx, unmatched) in unmatched_new_child_pass2
-                .iter()
-                .filter(|(idx, _, _)| *idx < new_idx)
-            {
-                if !already_inserted.contains(idx) {
-                    insert_node_patches.push(
-                        InsertNode::new(
-                            Some(&old_element.tag),
-                            *cur_node_idx,
-                            *new_node_idx,
-                            unmatched,
-                        )
-                        .into(),
-                    );
-                    already_inserted.push(*idx);
-                }
-            }
-
-            let mut this_new_child_node_idx = new_child_node_idx;
+            insert_node_patches.extend(create_insert_node_patches(
+                old_element,
+                &mut already_inserted,
+                &unmatched_new_child_pass2,
+                *cur_node_idx,
+                new_idx,
+                &child_cur_path,
+                &child_new_path,
+            ));
 
             matched_keyed_element_patches.extend(diff_recursive(
                 old_child,
                 new_child,
                 cur_node_idx,
-                &mut this_new_child_node_idx,
+                &mut new_child_node_idx,
+                &child_cur_path,
+                &child_new_path,
                 key,
                 skip,
                 rep,
             ));
         } else {
-            remove_node_patches
-                .push(RemoveNode::new(old_child.tag(), *cur_node_idx).into());
+            remove_node_patches.push(
+                RemoveNode::new(
+                    old_child.tag(),
+                    PatchPath::old(TreePath::start_at(
+                        *cur_node_idx,
+                        child_cur_path.clone(),
+                    )),
+                )
+                .into(),
+            );
             increment_node_idx_to_descendant_count(old_child, cur_node_idx);
         }
     }
 
     // the node that are to be appended are nodes
     // that are not matched, and not already part of the InsertNode
-    let mut append_children_patches = vec![];
+    let append_children_patches = create_append_children_patches(
+        old_element,
+        &mut already_inserted,
+        &unmatched_new_child_pass2,
+        &cur_path,
+        snapshot_cur_node_idx,
+    );
 
-    for (new_idx, new_node_idx, new_child) in unmatched_new_child_pass2.iter() {
-        if !already_inserted.contains(&new_idx) {
-            append_children_patches.push(
-                AppendChildren::new(
-                    &old_element.tag,
-                    new_child_excess_cur_node_idx,
-                    vec![(*new_node_idx, new_child)],
-                )
-                .into(),
-            );
-        }
-    }
+    let attributes_patches = create_attribute_patches(
+        old_element,
+        new_element,
+        *cur_node_idx,
+        *new_node_idx,
+        cur_path,
+        new_path,
+    );
 
     // patch order matters here
     // apply changes to the matched element first,
@@ -412,3 +468,93 @@ where
     patches.extend(remove_node_patches);
     patches
 }
+
+fn create_insert_node_patches<'a, NS, TAG, ATT, VAL, EVENT>(
+    old_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    already_inserted: &mut Vec<NodeIdx>,
+    unmatched_new_child_pass2: &Vec<(
+        usize,
+        usize,
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+    )>,
+    cur_node_idx: usize,
+    new_idx: usize,
+    child_cur_path: &Vec<usize>,
+    child_new_path: &Vec<usize>,
+) -> Vec<Patch<'a, NS, TAG, ATT, VAL, EVENT>>
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+    EVENT: PartialEq + Clone + Debug,
+{
+    let mut insert_node_patches = vec![];
+    for (idx, new_element_node_idx, unmatched) in unmatched_new_child_pass2
+        .iter()
+        .filter(|(idx, _, _)| *idx < new_idx)
+    {
+        if !already_inserted.contains(idx) {
+            insert_node_patches.push(
+                InsertNode::new(
+                    Some(&old_element.tag),
+                    PatchPath::new(
+                        TreePath::start_at(
+                            cur_node_idx,
+                            child_cur_path.clone(),
+                        ),
+                        TreePath::start_at(
+                            *new_element_node_idx,
+                            child_new_path.clone(),
+                        ),
+                    ),
+                    unmatched,
+                )
+                .into(),
+            );
+            already_inserted.push(*idx);
+        }
+    }
+
+    insert_node_patches
+}
+
+fn create_append_children_patches<'a, NS, TAG, ATT, VAL, EVENT>(
+    old_element: &'a Element<NS, TAG, ATT, VAL, EVENT>,
+    already_inserted: &mut Vec<NodeIdx>,
+    unmatched_new_child_pass2: &Vec<(
+        usize,
+        usize,
+        &'a Node<NS, TAG, ATT, VAL, EVENT>,
+    )>,
+    cur_path: &Vec<usize>,
+    snapshot_cur_node_idx: usize,
+) -> Vec<Patch<'a, NS, TAG, ATT, VAL, EVENT>>
+where
+    NS: PartialEq + Clone + Debug,
+    TAG: PartialEq + Clone + Debug,
+    ATT: PartialEq + Clone + Debug,
+    VAL: PartialEq + Clone + Debug,
+    EVENT: PartialEq + Clone + Debug,
+{
+    let mut append_children_patches = vec![];
+    for (new_idx, new_element_node_idx, new_child) in
+        unmatched_new_child_pass2.iter()
+    {
+        if !already_inserted.contains(&new_idx) {
+            append_children_patches.push(
+                AppendChildren::new(
+                    &old_element.tag,
+                    PatchPath::old(TreePath::start_at(
+                        snapshot_cur_node_idx,
+                        cur_path.clone(),
+                    )),
+                    vec![(*new_element_node_idx, new_child)],
+                )
+                .into(),
+            );
+            already_inserted.push(*new_idx);
+        }
+    }
+    append_children_patches
+}
