@@ -1,4 +1,4 @@
-/*#![deny(
+#![deny(
     warnings,
     missing_docs,
     missing_debug_implementations,
@@ -8,7 +8,6 @@
     unstable_features,
     unused_import_braces
 )]
-*/
 
 //! mt-dom is a generic virtual dom implementation which doesn't specify the types of the data that
 //! is being processed. It's up to the library user to specify those types
@@ -19,7 +18,6 @@
 //! mt-dom is not limited to be used in html base virtual-dom implementation, but can also be use
 //! for native UI elements.
 //!
-pub use apply_patches::apply_patches;
 pub use diff::diff_with_key;
 pub use node::{
     attribute::{
@@ -28,9 +26,9 @@ pub use node::{
     },
     element, element_ns, text, Attribute, Element, Node, Text,
 };
-pub use patch::{NodeIdx, Patch};
+pub use patch::{NodeIdx, Patch, PatchPath, TreePath};
 
-pub mod apply_patches;
 pub mod diff;
 mod node;
 pub mod patch;
+mod zipper;
