@@ -1,7 +1,6 @@
 //! patch is adding attributes
-use super::NodeIdx;
 use crate::Attribute;
-use std::fmt;
+use crate::PatchPath;
 use std::fmt::Debug;
 
 /// Add attributes
@@ -18,10 +17,8 @@ where
     /// use for verifying that the we are patching the correct node which
     /// should match the same tag
     pub tag: &'a TAG,
-    /// index of the node we are going to patch
-    pub node_idx: NodeIdx,
-    /// new node_idx of the node we are adding an attribute to
-    pub new_node_idx: NodeIdx,
+    /// the target dom traversal using this patch path
+    pub patch_path: PatchPath,
     /// the attributes to be patched into the target node
     pub attrs: Vec<&'a Attribute<NS, ATT, VAL, EVENT>>,
 }
@@ -39,14 +36,12 @@ where
     /// name are merged to produce a new unify attribute
     pub fn new(
         tag: &'a TAG,
-        node_idx: NodeIdx,
-        new_node_idx: NodeIdx,
+        patch_path: PatchPath,
         attrs: Vec<&'a Attribute<NS, ATT, VAL, EVENT>>,
     ) -> Self {
         AddAttributes {
             tag,
-            node_idx,
-            new_node_idx,
+            patch_path,
             attrs,
         }
     }
