@@ -1,6 +1,6 @@
 use super::NodeIdx;
 use crate::Node;
-use std::fmt;
+use crate::PatchPath;
 use std::fmt::Debug;
 
 /// Append a vector of child nodes to a parent node id.
@@ -16,7 +16,7 @@ where
     /// the tag of the node we are appending the children into
     pub tag: &'a TAG,
     /// index of the node we are going to append the children into
-    pub node_idx: NodeIdx,
+    pub patch_path: PatchPath,
     /// children nodes to be appended and their corresponding new_node_idx
     pub children: Vec<(NodeIdx, &'a Node<NS, TAG, ATT, VAL, EVENT>)>,
 }
@@ -32,12 +32,12 @@ where
     /// create a new AppendChildren patch
     pub fn new(
         tag: &'a TAG,
-        node_idx: NodeIdx,
+        patch_path: PatchPath,
         children: Vec<(NodeIdx, &'a Node<NS, TAG, ATT, VAL, EVENT>)>,
     ) -> Self {
         AppendChildren {
             tag,
-            node_idx,
+            patch_path,
             children,
         }
     }
