@@ -1,14 +1,11 @@
-use super::NodeIdx;
 use crate::node::Text;
+use crate::PatchPath;
 
 /// The patch is changing the text content of a text node
 #[derive(Clone, Debug, PartialEq)]
 pub struct ChangeText<'a> {
-    /// node index of the text node to be patch
-    /// relative to the root node of the application
-    pub node_idx: NodeIdx,
-    /// the new node_idx of this text
-    pub new_node_idx: NodeIdx,
+    /// the target element to be patch can be traverse using this patch path
+    pub patch_path: PatchPath,
     /// the old text is not really needed for applying the patch.
     /// but it is useful for debugging purposed, that we are changing the intended target text by
     /// visual inspection
@@ -19,16 +16,10 @@ pub struct ChangeText<'a> {
 
 impl<'a> ChangeText<'a> {
     /// create a new change text patch
-    pub fn new(
-        node_idx: NodeIdx,
-        old: &'a Text,
-        new_node_idx: NodeIdx,
-        new: &'a Text,
-    ) -> Self {
+    pub fn new(old: &'a Text, patch_path: PatchPath, new: &'a Text) -> Self {
         ChangeText {
-            node_idx,
+            patch_path,
             old,
-            new_node_idx,
             new,
         }
     }
