@@ -1,6 +1,5 @@
-use super::NodeIdx;
 use crate::Node;
-use std::fmt;
+use crate::PatchPath;
 use std::fmt::Debug;
 
 /// Replace a node with another node. This typically happens when a node's tag changes.
@@ -18,9 +17,7 @@ where
     /// can replace text node, and text node doesn't have tags
     pub tag: Option<&'a TAG>,
     /// the index of the node we are going to replace
-    pub node_idx: NodeIdx,
-    /// the node_idx of the replacement
-    pub new_node_idx: NodeIdx,
+    pub patch_path: PatchPath,
     /// the node that will replace the target node
     pub replacement: &'a Node<NS, TAG, ATT, VAL, EVENT>,
 }
@@ -36,14 +33,12 @@ where
     /// create a new ReplaceNode patch
     pub fn new(
         tag: Option<&'a TAG>,
-        node_idx: NodeIdx,
-        new_node_idx: NodeIdx,
+        patch_path: PatchPath,
         replacement: &'a Node<NS, TAG, ATT, VAL, EVENT>,
     ) -> Self {
         ReplaceNode {
             tag,
-            node_idx,
-            new_node_idx,
+            patch_path,
             replacement,
         }
     }
