@@ -39,7 +39,14 @@ fn key_1_removed_at_start() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
-    assert_eq!(diff, vec![RemoveNode::new(Some(&"div"), 1).into()]);
+    assert_eq!(
+        diff,
+        vec![RemoveNode::new(
+            Some(&"div"),
+            PatchPath::old(TreePath::start_at(1, vec![0, 0]),),
+        )
+        .into()]
+    );
 }
 
 #[test]
@@ -60,7 +67,14 @@ fn non_unique_keys_matched_at_old() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
-    assert_eq!(diff, vec![RemoveNode::new(Some(&"div"), 2).into()]);
+    assert_eq!(
+        diff,
+        vec![RemoveNode::new(
+            Some(&"div"),
+            PatchPath::old(TreePath::start_at(2, vec![0, 1]),),
+        )
+        .into()]
+    );
 }
 
 #[test]
@@ -81,7 +95,14 @@ fn key_2_removed_at_the_end() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
-    assert_eq!(diff, vec![RemoveNode::new(Some(&"div"), 2).into()]);
+    assert_eq!(
+        diff,
+        vec![RemoveNode::new(
+            Some(&"div"),
+            PatchPath::old(TreePath::start_at(2, vec![0, 1]),),
+        )
+        .into()]
+    );
 }
 
 #[test]
@@ -106,7 +127,14 @@ fn key_2_removed_at_the_middle() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
-    assert_eq!(diff, vec![RemoveNode::new(Some(&"div"), 2).into()]);
+    assert_eq!(
+        diff,
+        vec![RemoveNode::new(
+            Some(&"div"),
+            PatchPath::old(TreePath::start_at(2, vec![0, 1]),),
+        )
+        .into()]
+    );
 }
 
 #[test]
@@ -137,8 +165,20 @@ fn there_are_2_exact_same_keys_in_the_old() {
     assert_eq!(
         diff,
         vec![
-            ChangeText::new(2, &Text::new("0"), 2, &Text::new("1")).into(),
-            RemoveNode::new(Some(&"div"), 3).into()
+            ChangeText::new(
+                &Text::new("0"),
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0]),
+                    TreePath::start_at(2, vec![0, 0, 0])
+                ),
+                &Text::new("1")
+            )
+            .into(),
+            RemoveNode::new(
+                Some(&"div"),
+                PatchPath::old(TreePath::start_at(3, vec![0, 1]),),
+            )
+            .into()
         ]
     );
 }
@@ -171,11 +211,21 @@ fn there_are_2_exact_same_keys_in_the_new() {
     assert_eq!(
         diff,
         vec![
-            ChangeText::new(2, &Text::new("0"), 2, &Text::new("1")).into(),
+            ChangeText::new(
+                &Text::new("0"),
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0]),
+                    TreePath::start_at(2, vec![0, 0, 0])
+                ),
+                &Text::new("1")
+            )
+            .into(),
             InsertNode::new(
                 Some(&"main"),
-                3,
-                3,
+                PatchPath::new(
+                    TreePath::start_at(3, vec![0, 1]),
+                    TreePath::start_at(3, vec![0, 1])
+                ),
                 &element("div", vec![attr("key", "1")], vec![text(1)])
             )
             .into(),
@@ -212,16 +262,38 @@ fn there_are_2_exact_same_keys_in_both_old_and_new() {
     assert_eq!(
         diff,
         vec![
-            ChangeText::new(2, &Text::new("0"), 2, &Text::new("1")).into(),
-            ChangeText::new(4, &Text::new("1"), 6, &Text::new("3")).into(),
+            ChangeText::new(
+                &Text::new("0"),
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0]),
+                    TreePath::start_at(2, vec![0, 0, 0])
+                ),
+                &Text::new("1")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("1"),
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 1, 0]),
+                    TreePath::start_at(6, vec![0, 1, 0])
+                ),
+                &Text::new("3")
+            )
+            .into(),
             InsertNode::new(
                 Some(&"main"),
-                3,
-                3,
+                PatchPath::new(
+                    TreePath::start_at(3, vec![0, 1]),
+                    TreePath::start_at(3, vec![0, 1])
+                ),
                 &element("div", vec![attr("key", "1")], vec![text(2)])
             )
             .into(),
-            RemoveNode::new(Some(&"div"), 5).into(),
+            RemoveNode::new(
+                Some(&"div"),
+                PatchPath::old(TreePath::start_at(5, vec![0, 2]),),
+            )
+            .into(),
         ]
     );
 }
@@ -250,8 +322,10 @@ fn key_2_inserted_at_start() {
         diff,
         vec![InsertNode::new(
             Some(&"main"),
-            1,
-            1,
+            PatchPath::new(
+                TreePath::start_at(1, vec![0, 0]),
+                TreePath::start_at(1, vec![0, 0])
+            ),
             &element("div", vec![attr("key", "2")], vec![])
         )
         .into()]
@@ -279,8 +353,10 @@ fn keyed_element_not_reused() {
         diff,
         vec![ReplaceNode::new(
             Some(&"div"),
-            1,
-            1,
+            PatchPath::new(
+                TreePath::start_at(1, vec![0, 0]),
+                TreePath::start_at(1, vec![0, 0])
+            ),
             &element("div", vec![attr("key", "2")], vec![])
         )
         .into()]
@@ -312,13 +388,66 @@ fn key_2_inserted_at_the_end() {
         diff,
         vec![AppendChildren::new(
             &"main",
-            0,
+            PatchPath::old(TreePath::start_at(0, vec![0]),),
             vec![(2, &element("div", vec![attr("key", "2")], vec![]))]
         )
         .into()]
     );
 }
 
+#[test]
+fn test_append_at_sub_level() {
+    let old: MyNode = element(
+        "div",
+        vec![attr("id", "some-id"), attr("class", "some-class")],
+        vec![element(
+            "main",
+            vec![],
+            vec![element("div", vec![attr("key", "1")], vec![text(1)])],
+        )],
+    );
+
+    let new: MyNode = element(
+        "div",
+        vec![attr("id", "some-id"), attr("class", "some-class")],
+        vec![element(
+            "main",
+            vec![],
+            vec![
+                element("div", vec![attr("key", "1")], vec![text(1)]),
+                element("div", vec![attr("key", "2")], vec![text(2)]),
+                element("div", vec![attr("key", "3")], vec![text(3)]),
+            ],
+        )],
+    );
+
+    let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
+    assert_eq!(
+        diff,
+        vec![
+            AppendChildren::new(
+                &"main",
+                PatchPath::old(TreePath::start_at(1, vec![0, 0]),),
+                vec![(
+                    4,
+                    &element("div", vec![attr("key", "2")], vec![text(2)])
+                ),],
+            )
+            .into(),
+            AppendChildren::new(
+                &"main",
+                PatchPath::old(TreePath::start_at(1, vec![0, 0]),),
+                vec![(
+                    6,
+                    &element("div", vec![attr("key", "3")], vec![text(3)])
+                )],
+            )
+            .into()
+        ]
+    )
+}
+
 #[test]
 fn key_2_inserted_in_the_middle() {
     let old: MyNode = element(
@@ -348,8 +477,10 @@ fn key_2_inserted_in_the_middle() {
         diff,
         vec![InsertNode::new(
             Some(&"main"),
-            2,
-            2,
+            PatchPath::new(
+                TreePath::start_at(2, vec![0, 1]),
+                TreePath::start_at(2, vec![0, 1])
+            ),
             &element("div", vec![attr("key", "2")], vec![])
         )
         .into()]
@@ -378,18 +509,25 @@ fn key1_removed_at_start_then_key2_has_additional_attributes() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
     // we add attrubytes at NodeIdx 2, and this will become a NodeIdx 1
     assert_eq!(
         diff,
         vec![
             AddAttributes::new(
                 &"div",
-                2,
-                1,
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 1]),
+                    TreePath::start_at(1, vec![0, 1])
+                ),
                 vec![&attr("class", "some-class").into()]
             )
             .into(),
-            RemoveNode::new(Some(&"div"), 1).into(),
+            RemoveNode::new(
+                Some(&"div"),
+                PatchPath::old(TreePath::start_at(1, vec![0, 0]),),
+            )
+            .into(),
         ]
     );
 }
@@ -424,17 +562,24 @@ fn deep_nested_key1_removed_at_start_then_key2_has_additional_attributes() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
     assert_eq!(
         diff,
         vec![
             AddAttributes::new(
                 &"div",
-                3,
-                2,
+                PatchPath::new(
+                    TreePath::start_at(3, vec![0, 0, 1]),
+                    TreePath::start_at(2, vec![0, 0, 1])
+                ),
                 vec![&attr("class", "some-class").into()]
             )
             .into(),
-            RemoveNode::new(Some(&"div"), 2).into(),
+            RemoveNode::new(
+                Some(&"div"),
+                PatchPath::old(TreePath::start_at(2, vec![0, 0, 0]),),
+            )
+            .into(),
         ]
     );
 }
@@ -471,18 +616,29 @@ fn deep_nested_more_children_key0_and_key1_removed_at_start_then_key2_has_additi
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
     assert_eq!(
         diff,
         vec![
             AddAttributes::new(
                 &"div",
-                4,
-                2,
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 0, 2]),
+                    TreePath::start_at(2, vec![0, 0, 2])
+                ),
                 vec![&attr("class", "some-class").into()]
             )
             .into(),
-            RemoveNode::new(Some(&"div"), 2).into(),
-            RemoveNode::new(Some(&"div"), 3).into(),
+            RemoveNode::new(
+                Some(&"div"),
+                PatchPath::old(TreePath::start_at(2, vec![0, 0, 0]),),
+            )
+            .into(),
+            RemoveNode::new(
+                Some(&"div"),
+                PatchPath::old(TreePath::start_at(3, vec![0, 0, 1]),),
+            )
+            .into(),
         ]
     );
 }
@@ -540,32 +696,47 @@ fn deep_nested_keyed_with_non_keyed_children() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
     assert_eq!(
         diff,
         vec![
             AddAttributes::new(
                 &"div",
-                4,
-                2,
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 0, 2]),
+                    TreePath::start_at(2, vec![0, 0, 2])
+                ),
                 vec![&attr("class", "some-class").into()]
             )
             .into(),
             ChangeText::new(
-                6,
                 &Text::new("paragraph1"),
-                4,
+                PatchPath::new(
+                    TreePath::start_at(6, vec![0, 0, 2, 0, 0]),
+                    TreePath::start_at(4, vec![0, 0, 2, 0, 0])
+                ),
                 &Text::new("paragraph1, with added content")
             )
             .into(),
             ChangeText::new(
-                8,
                 &Text::new("Click here"),
-                6,
+                PatchPath::new(
+                    TreePath::start_at(8, vec![0, 0, 2, 1, 0]),
+                    TreePath::start_at(6, vec![0, 0, 2, 1, 0])
+                ),
                 &Text::new("Click here to continue")
             )
             .into(),
-            RemoveNode::new(Some(&"div"), 2).into(),
-            RemoveNode::new(Some(&"div"), 3).into(),
+            RemoveNode::new(
+                Some(&"div"),
+                PatchPath::old(TreePath::start_at(2, vec![0, 0, 0]),),
+            )
+            .into(),
+            RemoveNode::new(
+                Some(&"div"),
+                PatchPath::old(TreePath::start_at(3, vec![0, 0, 1]),),
+            )
+            .into(),
         ]
     );
 }
@@ -605,17 +776,25 @@ fn text_changed_in_keyed_elements() {
     );
 
     let patch = diff_with_key(&old, &update1, &"key");
+    dbg!(&patch);
+
     assert_eq!(
         patch,
         vec![
             ChangeText::new(
-                7,
                 &Text::new("item3"),
-                5,
+                PatchPath::new(
+                    TreePath::start_at(7, vec![0, 0, 2, 0]),
+                    TreePath::start_at(5, vec![0, 0, 2, 0])
+                ),
                 &Text::new("item3 with changes")
             )
             .into(),
-            RemoveNode::new(Some(&"article"), 2).into()
+            RemoveNode::new(
+                Some(&"article"),
+                PatchPath::old(TreePath::start_at(2, vec![0, 0, 0]),),
+            )
+            .into()
         ]
     );
 }
@@ -677,21 +856,30 @@ fn text_changed_in_mixed_keyed_and_non_keyed_elements() {
     );
 
     let patch = diff_with_key(&old, &update1, &"key");
+    dbg!(&patch);
     assert_eq!(
         patch,
         vec![
             ChangeText::new(
-                7,
                 &Text::new("item3"),
-                5,
+                PatchPath::new(
+                    TreePath::start_at(7, vec![0, 0, 2, 0]),
+                    TreePath::start_at(5, vec![0, 0, 2, 0])
+                ),
                 &Text::new("item3 with changes")
             )
             .into(),
-            RemoveNode::new(Some(&"article"), 2).into(),
+            RemoveNode::new(
+                Some(&"article"),
+                PatchPath::old(TreePath::start_at(2, vec![0, 0, 0]),),
+            )
+            .into(),
             ChangeText::new(
-                9,
                 &Text::new("3 items left"),
-                7,
+                PatchPath::new(
+                    TreePath::start_at(9, vec![0, 1, 0]),
+                    TreePath::start_at(7, vec![0, 1, 0])
+                ),
                 &Text::new("2 items left")
             )
             .into(),
@@ -759,21 +947,30 @@ fn test12() {
     );
 
     let patch = diff_with_key(&old, &update1, &"key");
+    dbg!(&patch);
     assert_eq!(
         patch,
         vec![
             ChangeText::new(
-                9,
                 &Text::new("item3"),
-                7,
+                PatchPath::new(
+                    TreePath::start_at(9, vec![0, 1, 2, 0]),
+                    TreePath::start_at(7, vec![0, 1, 2, 0])
+                ),
                 &Text::new("item3 with changes")
             )
             .into(),
-            RemoveNode::new(Some(&"article"), 4).into(),
+            RemoveNode::new(
+                Some(&"article"),
+                PatchPath::old(TreePath::start_at(4, vec![0, 1, 0]),),
+            )
+            .into(),
             ChangeText::new(
-                11,
                 &Text::new("3 items left"),
-                9,
+                PatchPath::new(
+                    TreePath::start_at(11, vec![0, 2, 0]),
+                    TreePath::start_at(9, vec![0, 2, 0])
+                ),
                 &Text::new("2 items left")
             )
             .into(),
@@ -803,5 +1000,13 @@ fn remove_first() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
-    assert_eq!(diff, vec![RemoveNode::new(Some(&"div"), 1).into()])
+    dbg!(&diff);
+    assert_eq!(
+        diff,
+        vec![RemoveNode::new(
+            Some(&"div"),
+            PatchPath::old(TreePath::start_at(1, vec![0, 0]),),
+        )
+        .into()]
+    )
 }
