@@ -12,7 +12,15 @@ fn test_replace_node() {
     let diff = diff_with_key(&old, &new, &"key");
     assert_eq!(
         diff,
-        vec![ReplaceNode::new(Some(&"div"), 0, 0, &new).into()],
+        vec![ReplaceNode::new(
+            Some(&"div"),
+            PatchPath::new(
+                TreePath::start_at(0, vec![0]),
+                TreePath::start_at(0, vec![0])
+            ),
+            &new
+        )
+        .into()],
     );
 }
 
@@ -22,7 +30,18 @@ fn test_replace_text_node() {
     let new = element("span", vec![], vec![]);
 
     let diff = diff_with_key(&old, &new, &"key");
-    assert_eq!(diff, vec![ReplaceNode::new(None, 0, 0, &new).into()],);
+    assert_eq!(
+        diff,
+        vec![ReplaceNode::new(
+            None,
+            PatchPath::new(
+                TreePath::start_at(0, vec![0]),
+                TreePath::start_at(0, vec![0])
+            ),
+            &new
+        )
+        .into()],
+    );
 }
 
 #[test]
@@ -36,8 +55,10 @@ fn test_replace_node_in_child() {
         diff,
         vec![ReplaceNode::new(
             Some(&"div"),
-            1,
-            1,
+            PatchPath::new(
+                TreePath::start_at(1, vec![0, 0]),
+                TreePath::start_at(1, vec![0, 0])
+            ),
             &element("span", vec![], vec![]).into()
         )
         .into()],
@@ -73,11 +94,22 @@ fn test_205() {
         ],
     ); //{ <div> <b> <i></i> </b> <i></i> </div>},
     assert_eq!(
-        diff_with_key(&old, &new, &"key"),
+        dbg!(diff_with_key(&old, &new, &"key")),
         vec![
-            RemoveNode::new(Some(&"i"), 3).into(),
-            ReplaceNode::new(Some(&"b"), 4, 3, &element("i", vec![], vec![]))
-                .into(),
+            RemoveNode::new(
+                Some(&"i"),
+                PatchPath::old(TreePath::start_at(3, vec![0, 0, 1]),),
+            )
+            .into(),
+            ReplaceNode::new(
+                Some(&"b"),
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 1]),
+                    TreePath::start_at(3, vec![0, 1])
+                ),
+                &element("i", vec![], vec![])
+            )
+            .into(),
         ],
     )
 }
@@ -137,8 +169,10 @@ fn test_class_changed() {
         diff,
         vec![AddAttributes::new(
             &"div",
-            0,
-            0,
+            PatchPath::new(
+                TreePath::start_at(0, vec![0]),
+                TreePath::start_at(0, vec![0])
+            ),
             vec![&attr("class", "some-class2")]
         )
         .into()]
@@ -160,12 +194,15 @@ fn text_node_changed() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
     assert_eq!(
         diff,
         vec![Patch::ChangeText(ChangeText::new(
-            1,
             &Text::new("text1"),
-            1,
+            PatchPath::new(
+                TreePath::start_at(1, vec![0, 0]),
+                TreePath::start_at(1, vec![0, 0])
+            ),
             &Text::new("text2")
         ))]
     )
@@ -186,8 +223,10 @@ fn test_class_will_not_be_merged_on_different_calls() {
         diff,
         vec![AddAttributes::new(
             &"div",
-            0,
-            0,
+            PatchPath::new(
+                TreePath::start_at(0, vec![0]),
+                TreePath::start_at(0, vec![0])
+            ),
             vec![&Attribute::with_multiple_values(
                 None,
                 "class",
@@ -213,8 +252,10 @@ fn test_class_removed() {
         diff,
         vec![RemoveAttributes::new(
             &"div",
-            0,
-            0,
+            PatchPath::new(
+                TreePath::start_at(0, vec![0]),
+                TreePath::start_at(0, vec![0])
+            ),
             vec![&attr("class", "some-class")]
         )
         .into()]
@@ -246,8 +287,10 @@ fn test_multiple_calls_to_style() {
         diff,
         vec![AddAttributes::new(
             &"div",
-            0,
-            0,
+            PatchPath::new(
+                TreePath::start_at(0, vec![0,]),
+                TreePath::start_at(0, vec![0,])
+            ),
             vec![
                 &attr("style", "display:flex"),
                 &attr("style", "width:200px;height:200px"),
@@ -269,8 +312,10 @@ fn inner_html_func_calls() {
         diff,
         vec![AddAttributes::new(
             &"div",
-            0,
-            0,
+            PatchPath::new(
+                TreePath::start_at(0, vec![0,]),
+                TreePath::start_at(0, vec![0,])
+            ),
             vec![&attr("inner_html", "<h1>Hello</h2>")]
         )
         .into()]
@@ -299,7 +344,7 @@ fn test_append() {
         diff,
         vec![AppendChildren::new(
             &"div",
-            0,
+            PatchPath::old(TreePath::start_at(0, vec![0]),),
             vec![(3, &element("div", vec![], vec![text(2)]))],
         )
         .into()]
@@ -329,7 +374,7 @@ fn test_append_more() {
         diff,
         vec![AppendChildren::new(
             &"div",
-            0,
+            PatchPath::old(TreePath::start_at(0, vec![0]),),
             vec![
                 (3, &element("div", vec![], vec![text(2)])),
                 (5, &element("div", vec![], vec![text(3)]))
@@ -338,3 +383,45 @@ fn test_append_more() {
         .into()]
     )
 }
+
+#[test]
+fn test_append_at_sub_level() {
+    let old: MyNode = element(
+        "div",
+        vec![attr("id", "some-id"), attr("class", "some-class")],
+        vec![element(
+            "main",
+            vec![],
+            vec![element("div", vec![], vec![text(1)])],
+        )],
+    );
+
+    let new: MyNode = element(
+        "div",
+        vec![attr("id", "some-id"), attr("class", "some-class")],
+        vec![element(
+            "main",
+            vec![],
+            vec![
+                element("div", vec![], vec![text(1)]),
+                element("div", vec![], vec![text(2)]),
+                element("div", vec![], vec![text(3)]),
+            ],
+        )],
+    );
+
+    let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
+    assert_eq!(
+        diff,
+        vec![AppendChildren::new(
+            &"main",
+            PatchPath::old(TreePath::start_at(1, vec![0, 0]),),
+            vec![
+                (4, &element("div", vec![], vec![text(2)])),
+                (6, &element("div", vec![], vec![text(3)]))
+            ],
+        )
+        .into()]
+    )
+}
