@@ -43,8 +43,10 @@ fn key_inserted_at_start() {
         diff,
         vec![InsertNode::new(
             Some(&"main"),
-            1,
-            1,
+            PatchPath::new(
+                TreePath::start_at(1, vec![0, 0]),
+                TreePath::start_at(1, vec![0, 0])
+            ),
             &element("div", vec![attr("key", "XXX")], vec![text("lineXXX")])
         )
         .into()]
@@ -88,12 +90,16 @@ fn key_inserted_at_middle() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
+
     assert_eq!(
         diff,
         vec![InsertNode::new(
             Some(&"main"),
-            11,
-            11,
+            PatchPath::new(
+                TreePath::start_at(11, vec![0, 5]),
+                TreePath::start_at(11, vec![0, 5])
+            ),
             &element("div", vec![attr("key", "XXX")], vec![text("lineXXX")])
         )
         .into()]
@@ -145,12 +151,15 @@ fn wrapped_elements() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
     assert_eq!(
         diff,
         vec![InsertNode::new(
             Some(&"main"),
-            12,
-            12,
+            PatchPath::new(
+                TreePath::start_at(12, vec![0, 0, 5]),
+                TreePath::start_at(12, vec![0, 0, 5])
+            ),
             &element("div", vec![attr("key", "XXX")], vec![text("lineXXX")])
         )
         .into()]
@@ -205,12 +214,15 @@ fn text_changed() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
     assert_eq!(
         diff,
         vec![ChangeText::new(
-            15,
             &Text::new("line7"),
-            15,
+            PatchPath::new(
+                TreePath::start_at(15, vec![0, 0, 6, 0]),
+                TreePath::start_at(15, vec![0, 0, 6, 0])
+            ),
             &Text::new("line7_changed")
         )
         .into()]
@@ -261,12 +273,16 @@ fn text_changed_non_keyed() {
     );
 
     let diff = diff_with_key(&old, &new, &"key");
+    dbg!(&diff);
+
     assert_eq!(
         diff,
         vec![ChangeText::new(
-            15,
             &Text::new("line7"),
-            15,
+            PatchPath::new(
+                TreePath::start_at(15, vec![0, 0, 6, 0]),
+                TreePath::start_at(15, vec![0, 0, 6, 0])
+            ),
             &Text::new("line7_changed")
         )
         .into()]
@@ -359,13 +375,39 @@ fn insert_one_line_at_start() {
     assert_eq!(
         diff,
         vec![
-            ChangeText::new(4, &Text::new("1"), 9, &Text::new("2")).into(),
-            ChangeText::new(9, &Text::new("2"), 14, &Text::new("3")).into(),
-            ChangeText::new(14, &Text::new("3"), 19, &Text::new("4")).into(),
+            ChangeText::new(
+                &Text::new("1"),
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 0, 0, 0, 0]),
+                    TreePath::start_at(9, vec![0, 0, 0, 0, 0])
+                ),
+                &Text::new("2")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("2"),
+                PatchPath::new(
+                    TreePath::start_at(9, vec![0, 0, 1, 0, 0]),
+                    TreePath::start_at(14, vec![0, 0, 1, 0, 0])
+                ),
+                &Text::new("3")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("3"),
+                PatchPath::new(
+                    TreePath::start_at(14, vec![0, 0, 2, 0, 0]),
+                    TreePath::start_at(19, vec![0, 0, 2, 0, 0])
+                ),
+                &Text::new("4")
+            )
+            .into(),
             InsertNode::new(
                 Some(&"main"),
-                2,
-                2,
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0]),
+                    TreePath::start_at(2, vec![0, 0, 0])
+                ),
                 &element(
                     "div",
                     vec![attr("key", "hashXXX")],
@@ -475,13 +517,39 @@ fn insert_two_lines_at_start() {
     assert_eq!(
         diff,
         vec![
-            ChangeText::new(4, &Text::new("1"), 14, &Text::new("3")).into(),
-            ChangeText::new(9, &Text::new("2"), 19, &Text::new("4")).into(),
-            ChangeText::new(14, &Text::new("2"), 24, &Text::new("5")).into(),
+            ChangeText::new(
+                &Text::new("1"),
+                PatchPath::new(
+                    TreePath::start_at(4, vec![0, 0, 0, 0, 0]),
+                    TreePath::start_at(14, vec![0, 0, 0, 0, 0])
+                ),
+                &Text::new("3")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("2"),
+                PatchPath::new(
+                    TreePath::start_at(9, vec![0, 0, 1, 0, 0]),
+                    TreePath::start_at(19, vec![0, 0, 1, 0, 0])
+                ),
+                &Text::new("4")
+            )
+            .into(),
+            ChangeText::new(
+                &Text::new("2"),
+                PatchPath::new(
+                    TreePath::start_at(14, vec![0, 0, 2, 0, 0]),
+                    TreePath::start_at(24, vec![0, 0, 2, 0, 0])
+                ),
+                &Text::new("5")
+            )
+            .into(),
             InsertNode::new(
                 Some(&"main"),
-                2,
-                2,
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0]),
+                    TreePath::start_at(2, vec![0, 0, 0])
+                ),
                 &element(
                     "div",
                     vec![attr("key", "hashXXX")],
@@ -494,8 +562,10 @@ fn insert_two_lines_at_start() {
             .into(),
             InsertNode::new(
                 Some(&"main"),
-                2,
-                7,
+                PatchPath::new(
+                    TreePath::start_at(2, vec![0, 0, 0]),
+                    TreePath::start_at(7, vec![0, 0, 0])
+                ),
                 &element(
                     "div",
                     vec![attr("key", "hashYYY")],
